@@ -1,5 +1,5 @@
 /* GObject - GLib Type, Object, Parameter and Signal Library
- * Copyright (C) 2000 Red Hat, Inc.
+ * Copyright (C) 2000-2001 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,16 +20,20 @@
 
 #include	"gbsearcharray.h"
 #include	"gvalue.h"
+#include	"gvaluearray.h"
+#include	"gclosure.h"
 #include	"gvaluecollector.h"
 
-
+#include <string.h>
 
 /* --- typedefs & structures --- */
 typedef struct
 {
   GType		 type;
+  GBoxedInitFunc init;
   GBoxedCopyFunc copy;
   GBoxedFreeFunc free;
+  gboolean	 is_refcounted;
 } BoxedNode;
 
 
@@ -39,7 +43,7 @@ static gint	boxed_nodes_cmp		(gconstpointer	p1,
 
 
 /* --- variables --- */
-static GBSearchArray boxed_bsa = { boxed_nodes_cmp, sizeof (BoxedNode), 0, 0, NULL };
+static GBSearchArray boxed_bsa = G_STATIC_BSEARCH_ARRAY_INIT (sizeof (BoxedNode), boxed_nodes_cmp, 0);
 
 
 /* --- functions --- */
@@ -52,6 +56,66 @@ boxed_nodes_cmp	(gconstpointer p1,
   return G_BSEARCH_ARRAY_CMP (node1->type, node2->type);
 }
 
+static inline void              /* keep this function in sync with gvalue.c */
+value_meminit (GValue *value,
+	       GType   value_type)
+{
+  value->g_type = value_type;
+  memset (value->data, 0, sizeof (value->data));
+}
+
+static gpointer
+value_copy (gpointer boxed)
+{
+  const GValue *src_value = boxed;
+  GValue *dest_value = g_new0 (GValue, 1);
+
+  if (G_VALUE_TYPE (src_value))
+    {
+      g_value_init (dest_value, G_VALUE_TYPE (src_value));
+      g_value_copy (src_value, dest_value);
+    }
+  return dest_value;
+}
+
+static void
+value_free (gpointer boxed)
+{
+  GValue *value = boxed;
+
+  if (G_VALUE_TYPE (value))
+    g_value_unset (value);
+  g_free (value);
+}
+
+static gpointer
+value_array_init (void)
+{
+  return g_value_array_new (0);
+}
+
+static gpointer
+gstring_init (void)
+{
+  return g_string_new ("");
+}
+
+static gpointer
+gstring_copy (gpointer boxed)
+{
+  const GString *src_gstring = boxed;
+
+  return g_string_new_len (src_gstring->str, src_gstring->len);
+}
+
+static void
+gstring_free (gpointer boxed)
+{
+  GString *gstring = boxed;
+
+  g_string_free (gstring, TRUE);
+}
+
 void
 g_boxed_type_init (void)  /* sync with gtype.c */
 {
@@ -72,24 +136,66 @@ g_boxed_type_init (void)  /* sync with gtype.c */
 
   /* G_TYPE_BOXED
    */
-  type = g_type_register_fundamental (G_TYPE_BOXED, "GBoxed", &info, &finfo, G_TYPE_FLAG_ABSTRACT);
+  type = g_type_register_fundamental (G_TYPE_BOXED, "GBoxed", &info, &finfo,
+				      G_TYPE_FLAG_ABSTRACT | G_TYPE_FLAG_VALUE_ABSTRACT);
   g_assert (type == G_TYPE_BOXED);
+
+  /* boxed: G_TYPE_CLOSURE
+   */
+  type = g_boxed_type_register_static ("GClosure",
+				       (GBoxedInitFunc) NULL,
+				       (GBoxedCopyFunc) g_closure_ref,
+				       (GBoxedFreeFunc) g_closure_unref,
+				       TRUE);
+  g_assert (type == G_TYPE_CLOSURE);
+
+  /* boxed: G_TYPE_VALUE
+   */
+  type = g_boxed_type_register_static ("GValue",
+				       (GBoxedInitFunc) NULL,
+				       value_copy,
+				       value_free,
+				       FALSE);
+  g_assert (type == G_TYPE_VALUE);
+
+  /* boxed: G_TYPE_VALUE_ARRAY
+   */
+  type = g_boxed_type_register_static ("GValueArray",
+				       value_array_init,	/* don't allow NULL values */
+				       (GBoxedCopyFunc) g_value_array_copy,
+				       (GBoxedFreeFunc) g_value_array_free,
+				       FALSE);
+  g_assert (type == G_TYPE_VALUE_ARRAY);
+
+  /* boxed: G_TYPE_GSTRING
+   * yes, the naming is a bit odd, but GString is obviously not G_TYPE_STRING
+   */
+  type = g_boxed_type_register_static ("GString",
+				       gstring_init,		/* don't allow NULL values */
+				       gstring_copy,
+				       gstring_free,
+				       FALSE);
+  g_assert (type == G_TYPE_GSTRING);
 }
 
 static void
 boxed_proxy_value_init (GValue *value)
 {
-  value->data[0].v_pointer = 0;
+  BoxedNode key, *node;
+
+  key.type = G_VALUE_TYPE (value);
+  node = g_bsearch_array_lookup (&boxed_bsa, &key);
+  value->data[0].v_pointer = node->init ? node->init () : NULL;
 }
 
 static void
 boxed_proxy_value_free (GValue *value)
 {
-  if (value->data[0].v_pointer && !(value->data[1].v_uint & G_VALUE_STATIC_TAG))
+  if (value->data[0].v_pointer && !(value->data[1].v_uint & G_VALUE_NOCOPY_CONTENTS))
     {
       BoxedNode key, *node;
 
-      key.type = value->g_type;
+      key.type = G_VALUE_TYPE (value);
       node = g_bsearch_array_lookup (&boxed_bsa, &key);
       node->free (value->data[0].v_pointer);
     }
@@ -103,7 +209,7 @@ boxed_proxy_value_copy (const GValue *src_value,
     {
       BoxedNode key, *node;
 
-      key.type = src_value->g_type;
+      key.type = G_VALUE_TYPE (src_value);
       node = g_bsearch_array_lookup (&boxed_bsa, &key);
       dest_value->data[0].v_pointer = node->copy (src_value->data[0].v_pointer);
     }
@@ -119,53 +225,75 @@ boxed_proxy_value_peek_pointer (const GValue *value)
 
 static gchar*
 boxed_proxy_collect_value (GValue      *value,
-			   guint        nth_value,
-			   GType       *collect_type,
-			   GTypeCValue *collect_value)
+			   guint        n_collect_values,
+			   GTypeCValue *collect_values,
+			   guint        collect_flags)
 {
   BoxedNode key, *node;
 
-  key.type = value->g_type;
+  key.type = G_VALUE_TYPE (value);
   node = g_bsearch_array_lookup (&boxed_bsa, &key);
-  value->data[0].v_pointer = node->copy (collect_value->v_pointer);
 
-  *collect_type = 0;
+  /* for NULL values, we have to call GBoxedInitFunc */
+  if (!collect_values[0].v_pointer)
+    value->data[0].v_pointer = node->init ? node->init () : NULL;
+  else
+    {
+      /* never honour G_VALUE_NOCOPY_CONTENTS for ref-counted types */
+      if (!node->is_refcounted && (collect_flags & G_VALUE_NOCOPY_CONTENTS))
+	{
+	  value->data[0].v_pointer = collect_values[0].v_pointer;
+	  value->data[1].v_uint = G_VALUE_NOCOPY_CONTENTS;
+	}
+      else
+	value->data[0].v_pointer = node->copy (collect_values[0].v_pointer);
+    }
+
   return NULL;
 }
 
 static gchar*
 boxed_proxy_lcopy_value (const GValue *value,
-			 guint         nth_value,
-			 GType        *collect_type,
-			 GTypeCValue  *collect_value)
+			 guint         n_collect_values,
+			 GTypeCValue  *collect_values,
+			 guint         collect_flags)
 {
-  BoxedNode key, *node;
-  gpointer *boxed_p = collect_value->v_pointer;
+  gpointer *boxed_p = collect_values[0].v_pointer;
 
   if (!boxed_p)
     return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
 
-  key.type = value->g_type;
-  node = g_bsearch_array_lookup (&boxed_bsa, &key);
-  *boxed_p = node->copy (value->data[0].v_pointer);
+  if (!value->data[0].v_pointer)
+    *boxed_p = NULL;
+  else if (collect_flags & G_VALUE_NOCOPY_CONTENTS)
+    *boxed_p = value->data[0].v_pointer;
+  else
+    {
+      BoxedNode key, *node;
+
+      key.type = G_VALUE_TYPE (value);
+      node = g_bsearch_array_lookup (&boxed_bsa, &key);
+      *boxed_p = node->copy (value->data[0].v_pointer);
+    }
 
-  *collect_type = 0;
   return NULL;
 }
 
 GType
 g_boxed_type_register_static (const gchar   *name,
+			      GBoxedInitFunc boxed_init,
 			      GBoxedCopyFunc boxed_copy,
-			      GBoxedFreeFunc boxed_free)
+			      GBoxedFreeFunc boxed_free,
+			      gboolean	     is_refcounted)
 {
   static const GTypeValueTable vtable = {
     boxed_proxy_value_init,
     boxed_proxy_value_free,
     boxed_proxy_value_copy,
     boxed_proxy_value_peek_pointer,
-    G_VALUE_COLLECT_POINTER,
+    "p",
     boxed_proxy_collect_value,
-    G_VALUE_COLLECT_POINTER,
+    "p",
     boxed_proxy_lcopy_value,
   };
   static const GTypeInfo type_info = {
@@ -195,15 +323,17 @@ g_boxed_type_register_static (const gchar   *name,
       BoxedNode key;
 
       key.type = type;
+      key.init = boxed_init;
       key.copy = boxed_copy;
       key.free = boxed_free;
+      key.is_refcounted = is_refcounted != FALSE;
       g_bsearch_array_insert (&boxed_bsa, &key, TRUE);
     }
 
   return type;
 }
 
-GBoxed*
+gpointer
 g_boxed_copy (GType         boxed_type,
 	      gconstpointer src_boxed)
 {
@@ -231,14 +361,26 @@ g_boxed_copy (GType         boxed_type,
     {
       GValue src_value, dest_value;
       
-      /* we heavil rely on the gvalue.c implementation here */
-
-      memset (&src_value.data, 0, sizeof (src_value.data));
-      memset (&dest_value.data, 0, sizeof (dest_value.data));
-      dest_value.g_type = boxed_type;
-      src_value.g_type = boxed_type;
+      /* we heavil rely on third-party boxed type value vtable
+       * implementations to follow normal boxed value storage
+       * (data[0].v_pointer is the boxed struct, and
+       * data[1].v_uint holds the G_VALUE_NOCOPY_CONTENTS flag,
+       * rest zero).
+       * but then, we can expect that since we layed out the
+       * g_boxed_*() API.
+       * data[1].v_uint&G_VALUE_NOCOPY_CONTENTS shouldn't be set
+       * after a copy.
+       */
+      /* equiv. to g_value_set_static_boxed() */
+      value_meminit (&src_value, boxed_type);
       src_value.data[0].v_pointer = (gpointer) src_boxed;
+      src_value.data[1].v_uint = G_VALUE_NOCOPY_CONTENTS;
+
+      /* call third-party code copy fucntion, fingers-crossed */
+      value_meminit (&dest_value, boxed_type);
       value_table->value_copy (&src_value, &dest_value);
+
+      /* double check and grouse if things went wrong */
       if (dest_value.data[1].v_ulong ||
 	  dest_value.data[2].v_ulong ||
 	  dest_value.data[3].v_ulong)
@@ -278,54 +420,100 @@ g_boxed_free (GType    boxed_type,
     {
       GValue value;
 
-      /* we heavil rely on the gvalue.c implementation here */
-      memset (&value.data, 0, sizeof (value.data));
-      value.g_type = boxed_type;
+      /* see g_boxed_copy() on why we think we can do this */
+      value_meminit (&value, boxed_type);
       value.data[0].v_pointer = boxed;
       value_table->value_free (&value);
     }
 }
 
+gpointer
+g_value_get_boxed (const GValue *value)
+{
+  g_return_val_if_fail (G_VALUE_HOLDS_BOXED (value), NULL);
+  g_return_val_if_fail (G_TYPE_IS_VALUE (G_VALUE_TYPE (value)), NULL);
+
+  return value->data[0].v_pointer;
+}
+
+gpointer
+g_value_dup_boxed (const GValue *value)
+{
+  g_return_val_if_fail (G_VALUE_HOLDS_BOXED (value), NULL);
+  g_return_val_if_fail (G_TYPE_IS_VALUE (G_VALUE_TYPE (value)), NULL);
+
+  return value->data[0].v_pointer ? g_boxed_copy (G_VALUE_TYPE (value), value->data[0].v_pointer) : NULL;
+}
+
+static inline void
+value_set_boxed_internal (GValue       *value,
+			  gconstpointer const_boxed,
+			  gboolean      need_copy,
+			  gboolean      need_free)
+{
+  BoxedNode key, *node;
+  gpointer boxed = (gpointer) const_boxed;
+
+  if (!boxed)
+    {
+      /* just resetting to NULL might not be desired, need to
+       * have value reinitialized also (for values defaulting
+       * to other default value states than a NULL data pointer),
+       * g_value_reset() will handle this
+       */
+      g_value_reset (value);
+      return;
+    }
+
+  key.type = G_VALUE_TYPE (value);
+  node = g_bsearch_array_lookup (&boxed_bsa, &key);
+
+  if (node)
+    {
+      /* we proxy this type, free contents and copy right away */
+      if (value->data[0].v_pointer && !(value->data[1].v_uint & G_VALUE_NOCOPY_CONTENTS))
+	node->free (value->data[0].v_pointer);
+      value->data[1].v_uint = need_free ? 0 : G_VALUE_NOCOPY_CONTENTS;
+      value->data[0].v_pointer = need_copy ? node->copy (boxed) : boxed;
+    }
+  else
+    {
+      /* we don't handle this type, free contents and let g_boxed_copy()
+       * figure what's required
+       */
+      if (value->data[0].v_pointer && !(value->data[1].v_uint & G_VALUE_NOCOPY_CONTENTS))
+	g_boxed_free (G_VALUE_TYPE (value), value->data[0].v_pointer);
+      value->data[1].v_uint = need_free ? 0 : G_VALUE_NOCOPY_CONTENTS;
+      value->data[0].v_pointer = need_copy ? g_boxed_copy (G_VALUE_TYPE (value), boxed) : boxed;
+    }
+}
+
 void
 g_value_set_boxed (GValue       *value,
 		   gconstpointer boxed)
 {
-  g_return_if_fail (G_IS_VALUE_BOXED (value));
+  g_return_if_fail (G_VALUE_HOLDS_BOXED (value));
   g_return_if_fail (G_TYPE_IS_VALUE (G_VALUE_TYPE (value)));
 
-  if (value->data[0].v_pointer && !(value->data[1].v_uint & G_VALUE_STATIC_TAG))
-    g_boxed_free (G_VALUE_TYPE (value), value->data[0].v_pointer);
-  value->data[0].v_pointer = boxed ? g_boxed_copy (G_VALUE_TYPE (value), boxed) : NULL;
-  value->data[1].v_uint = 0;
+  value_set_boxed_internal (value, boxed, TRUE, TRUE);
 }
 
 void
 g_value_set_static_boxed (GValue       *value,
 			  gconstpointer boxed)
 {
-  g_return_if_fail (G_IS_VALUE_BOXED (value));
+  g_return_if_fail (G_VALUE_HOLDS_BOXED (value));
   g_return_if_fail (G_TYPE_IS_VALUE (G_VALUE_TYPE (value)));
 
-  if (value->data[0].v_pointer && !(value->data[1].v_uint & G_VALUE_STATIC_TAG))
-    g_boxed_free (G_VALUE_TYPE (value), value->data[0].v_pointer);
-  value->data[0].v_pointer = (gpointer) boxed;
-  value->data[1].v_uint = boxed ? G_VALUE_STATIC_TAG : 0;
+  value_set_boxed_internal (value, boxed, FALSE, FALSE);
 }
 
-gpointer
-g_value_get_boxed (const GValue *value)
+void
+g_value_set_boxed_take_ownership (GValue       *value,
+				  gconstpointer boxed)
 {
-  g_return_val_if_fail (G_IS_VALUE_BOXED (value), NULL);
-  g_return_val_if_fail (G_TYPE_IS_VALUE (G_VALUE_TYPE (value)), NULL);
+  g_return_if_fail (G_VALUE_HOLDS_BOXED (value));
+  g_return_if_fail (G_TYPE_IS_VALUE (G_VALUE_TYPE (value)));
 
-  return value->data[0].v_pointer;
-}
-
-gpointer
-g_value_dup_boxed (GValue *value)
-{
-  g_return_val_if_fail (G_IS_VALUE_BOXED (value), NULL);
-  g_return_val_if_fail (G_TYPE_IS_VALUE (G_VALUE_TYPE (value)), NULL);
-
-  return value->data[0].v_pointer ? g_boxed_copy (G_VALUE_TYPE (value), value->data[0].v_pointer) : NULL;
+  value_set_boxed_internal (value, boxed, FALSE, TRUE);
 }
