@@ -1,5 +1,5 @@
 /* GLIB-GenMarshal - Marshaller generator for GObject library
- * Copyright (C) 2000 Red Hat, Inc.
+ * Copyright (C) 2000-2001 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,13 @@
  */
 #include	"config.h"
 
-#include	<glib-object.h>
+
+/* ok, this is a bit hackish, have to provide gruntime log domain as
+ * we don't link against -lgruntime
+ */
+char *g_log_domain_gruntime = "GLib-Genmarshal";
+
+#include	<glib.h>
 
 #include	<stdio.h>
 #include	<stdlib.h>
@@ -42,21 +48,28 @@
 
 
 /* --- typedefs & structures --- */
-typedef struct _Argument  Argument;
-typedef struct _Signature Signature;
-struct _Argument
+typedef struct
 {
-  gchar       *pname;		/* parsed name */
-  const gchar *sname;		/* signature name */
-  const gchar *func;		/* functional extension */
-  const gchar *cname;		/* C name */
-};
-struct _Signature
+  gchar	      *keyword;		/* marhaller list keyword [MY_STRING] */
+  const gchar *sig_name;	/* signature name [STRING] */
+  const gchar *ctype;		/* C type name [gchar*] */
+  const gchar *getter;		/* value getter function [g_value_get_string] */
+} InArgument;
+typedef struct
 {
-  gchar    *ploc;
-  Argument *rarg;
-  GList    *args;	/* of type Argument* */
-};
+  gchar	      *keyword;		/* marhaller list keyword [MY_STRING] */
+  const gchar *sig_name;	/* signature name [STRING] */
+  const gchar *ctype;		/* C type name [gchar*] */
+  const gchar *setter;		/* value setter function [g_value_set_string] */
+  const gchar *release;		/* value release function [g_free] */
+  const gchar *release_check;   /* checks if release function is safe to call */
+} OutArgument;
+typedef struct
+{
+  gchar       *ploc;
+  OutArgument *rarg;
+  GList       *args;	/* of type InArgument* */
+} Signature;
 
 
 /* --- prototypes --- */
@@ -108,79 +121,101 @@ static GScannerConfig scanner_config_template =
   FALSE                 /* symbol_2_token */,
   FALSE                 /* scope_0_fallback */,
 };
-static gchar		*marshaller_prefix = "g_cclosure_marshal";
+static gchar		*std_marshaller_prefix = "g_cclosure_marshal";
+static gchar		*marshaller_prefix = "g_cclosure_user_marshal";
 static GHashTable	*marshallers = NULL;
 static gboolean		 gen_cheader = FALSE;
 static gboolean		 gen_cbody = FALSE;
 static gboolean		 skip_ploc = FALSE;
+static gboolean		 std_includes = TRUE;
 
 
 /* --- functions --- */
 static gboolean
-complete_arg (Argument *arg,
-	      gboolean  is_return)
+complete_in_arg (InArgument *iarg)
 {
-  static const Argument inout_arguments[] = {
-    /* pname,		sname,		func,			cname		*/
-    { "VOID",		"VOID",		NULL,			"void",		},
-    { "BOOLEAN",	"BOOLEAN",	"boolean",		"gboolean",	},
-    { "CHAR",		"CHAR",		"char",			"gchar",	},
-    { "UCHAR",		"UCHAR",	"uchar",		"guchar",	},
-    { "INT",		"INT",		"int",			"gint",		},
-    { "UINT",		"UINT",		"uint",			"guint",	},
-    { "LONG",		"LONG",		"long",			"glong",	},
-    { "ULONG",		"ULONG",	"ulong",		"gulong",	},
-    { "ENUM",		"ENUM",		"enum",			"gint",		},
-    { "FLAGS",		"FLAGS",	"flags",		"guint",	},
-    { "FLOAT",		"FLOAT",	"float",		"gfloat",	},
-    { "DOUBLE",		"DOUBLE",	"double",		"gdouble",	},
+  static const InArgument args[] = {
+    /* keyword		sig_name	ctype		getter			*/
+    { "VOID",		"VOID",		"void",		NULL,			},
+    { "BOOLEAN",	"BOOLEAN",	"gboolean",	"g_value_get_boolean",	},
+    { "CHAR",		"CHAR",		"gchar",	"g_value_get_char",	},
+    { "UCHAR",		"UCHAR",	"guchar",	"g_value_get_uchar",	},
+    { "INT",		"INT",		"gint",		"g_value_get_int",	},
+    { "UINT",		"UINT",		"guint",	"g_value_get_uint",	},
+    { "LONG",		"LONG",		"glong",	"g_value_get_long",	},
+    { "ULONG",		"ULONG",	"gulong",	"g_value_get_ulong",	},
+    { "ENUM",		"ENUM",		"gint",		"g_value_get_enum",	},
+    { "FLAGS",		"FLAGS",	"guint",	"g_value_get_flags",	},
+    { "FLOAT",		"FLOAT",	"gfloat",	"g_value_get_float",	},
+    { "DOUBLE",		"DOUBLE",	"gdouble",	"g_value_get_double",	},
+    { "STRING",		"STRING",	"gpointer",	"(char*) g_value_get_string",	},
+    { "PARAM",		"PARAM",	"gpointer",	"g_value_get_param",	},
+    { "BOXED",		"BOXED",	"gpointer",	"g_value_get_boxed",	},
+    { "POINTER",	"POINTER",	"gpointer",	"g_value_get_pointer",	},
+    { "OBJECT",		"OBJECT",	"gpointer",	"g_value_get_object",	},
     /* deprecated: */
-    { "NONE",		"VOID",		NULL,			"void",		},
-    { "BOOL",		"BOOLEAN",	"boolean",		"gboolean",	},
+    { "NONE",		"VOID",		"void",		NULL,			},
+    { "BOOL",		"BOOLEAN",	"gboolean",	"g_value_get_boolean",	},
   };
-  static const Argument in_arguments[] = {
-    { "STRING",		"POINTER",	"as_pointer",		"gpointer",	},
-    { "BOXED",		"POINTER",	"as_pointer",		"gpointer",	},
-    { "POINTER",	"POINTER",	"as_pointer",		"gpointer",	},
-    { "OBJECT",		"POINTER",	"as_pointer",		"gpointer",	},
-  };
-  static const Argument out_arguments[] = {
-    { "STRING",		"STRING",	"string",		"gchar*",	},
-    { "BOXED",		"BOXED",	"boxed",		"gpointer",	},
-    { "POINTER",	"POINTER",	"pointer",		"gpointer",	},
-    { "OBJECT",		"OBJECT",	"object",		"GObject*",	},
-  };
-  const guint n_inout_arguments = sizeof (inout_arguments) / sizeof (inout_arguments[0]);
-  const guint n_out_arguments = sizeof (out_arguments) / sizeof (out_arguments[0]);
-  const guint n_in_arguments = sizeof (in_arguments) / sizeof (in_arguments[0]);
-  const Argument *arguments;
-  guint i, n_arguments;
+  const guint n_args = sizeof (args) / sizeof (args[0]);
+  guint i;
 
-  g_return_val_if_fail (arg != NULL, FALSE);
+  g_return_val_if_fail (iarg != NULL, FALSE);
 
-  arguments = inout_arguments;
-  n_arguments = n_inout_arguments;
-  for (i = 0; i < n_arguments; i++)
-    if (strcmp (arguments[i].pname, arg->pname) == 0)
+  for (i = 0; i < n_args; i++)
+    if (strcmp (args[i].keyword, iarg->keyword) == 0)
       {
-	arg->sname = arguments[i].sname;
-	arg->func = arguments[i].func;
-	arg->cname = arguments[i].cname;
+	iarg->sig_name = args[i].sig_name;
+	iarg->ctype = args[i].ctype;
+	iarg->getter = args[i].getter;
 
 	return TRUE;
       }
-  arguments = is_return ? out_arguments : in_arguments;
-  n_arguments = is_return ? n_out_arguments : n_in_arguments;
-  for (i = 0; i < n_arguments; i++)
-    if (strcmp (arguments[i].pname, arg->pname) == 0)
+  return FALSE;
+}
+
+static gboolean
+complete_out_arg (OutArgument *oarg)
+{
+  static const OutArgument args[] = {
+    /* keyword		sig_name	ctype		setter			release                 release_check */
+    { "VOID",		"VOID",		"void",		NULL,			NULL,			NULL          },
+    { "BOOLEAN",	"BOOLEAN",	"gboolean",	"g_value_set_boolean",	NULL,			NULL          },
+    { "CHAR",		"CHAR",		"gchar",	"g_value_set_char",	NULL,			NULL          },
+    { "UCHAR",		"UCHAR",	"guchar",	"g_value_set_uchar",	NULL,			NULL          },
+    { "INT",		"INT",		"gint",		"g_value_set_int",	NULL,			NULL          },
+    { "UINT",		"UINT",		"guint",	"g_value_set_uint",	NULL,			NULL          },
+    { "LONG",		"LONG",		"glong",	"g_value_set_long",	NULL,			NULL          },
+    { "ULONG",		"ULONG",	"gulong",	"g_value_set_ulong",	NULL,			NULL          },
+    { "ENUM",		"ENUM",		"gint",		"g_value_set_enum",	NULL,			NULL          },
+    { "FLAGS",		"FLAGS",	"guint",	"g_value_set_flags",	NULL,			NULL          },
+    { "FLOAT",		"FLOAT",	"gfloat",	"g_value_set_float",	NULL,			NULL          },
+    { "DOUBLE",		"DOUBLE",	"gdouble",	"g_value_set_double",	NULL,			NULL          },
+    { "STRING",		"STRING",	"gchar*",	"g_value_set_string_take_ownership", NULL,	NULL          },
+    { "PARAM",		"PARAM",	"GParamSpec*",	"g_value_set_param",	"g_param_spec_unref",	NULL          },
+    { "BOXED",		"BOXED",	"gpointer",	"g_value_set_boxed_take_ownership", NULL,	NULL          },
+    { "POINTER",	"POINTER",	"gpointer",	"g_value_set_pointer",	NULL,			NULL          },
+    { "OBJECT",		"OBJECT",	"GObject*",	"g_value_set_object",	"g_object_unref",	"NULL !="     },
+    /* deprecated: */
+    { "NONE",		"VOID",		"void",		NULL,			NULL,			NULL          },
+    { "BOOL",		"BOOLEAN",	"gboolean",	"g_value_set_boolean",	NULL,			NULL          }
+  };
+  const guint n_args = sizeof (args) / sizeof (args[0]);
+  guint i;
+
+  g_return_val_if_fail (oarg != NULL, FALSE);
+
+  for (i = 0; i < n_args; i++)
+    if (strcmp (args[i].keyword, oarg->keyword) == 0)
       {
-	arg->sname = arguments[i].sname;
-	arg->func = arguments[i].func;
-	arg->cname = arguments[i].cname;
+	oarg->sig_name = args[i].sig_name;
+	oarg->ctype = args[i].ctype;
+	oarg->setter = args[i].setter;
+	oarg->release = args[i].release;
+	oarg->release_check = args[i].release_check;
 
 	return TRUE;
       }
-
   return FALSE;
 }
 
@@ -201,7 +236,7 @@ pad (const gchar *string)
     {
       g_free (buffer);
       buffer = g_strdup_printf ("%s ", string);
-      g_warning ("overfull string (%u bytes) for padspace", strlen (string));
+      g_warning ("overfull string (%u bytes) for padspace", (guint) strlen (string));
 
       return buffer;
     }
@@ -220,7 +255,7 @@ pad (const gchar *string)
 static const gchar*
 indent (guint n_spaces)
 {
-  static gchar *buffer;
+  static gchar *buffer = NULL;
   static guint blength = 0;
 
   if (blength <= n_spaces)
@@ -241,17 +276,39 @@ generate_marshal (const gchar *signame,
 {
   guint ind, a;
   GList *node;
+  gchar *tmp = g_strconcat (marshaller_prefix, "_", signame, NULL);
+  gboolean have_std_marshaller = FALSE;
 
-  if (g_hash_table_lookup (marshallers, signame))
-    return;
+  /* here we have to make sure a marshaller named <marshaller_prefix>_<signame>
+   * exists. we might have put it out already, can revert to a standard marshaller
+   * provided by glib, or need to generate one.
+   */
+
+  if (g_hash_table_lookup (marshallers, tmp))
+    {
+      /* done, marshaller already generated */
+      g_free (tmp);
+      return;
+    }
   else
     {
-      gchar *tmp = g_strdup (signame);
-
+      /* need to alias/generate marshaller, register name */
       g_hash_table_insert (marshallers, tmp, tmp);
     }
-  
-  if (gen_cheader)
+
+  /* can we revert to a standard marshaller? */
+  if (std_includes)
+    {
+      tmp = g_strconcat (std_marshaller_prefix, "_", signame, NULL);
+      have_std_marshaller = g_hash_table_lookup (marshallers, tmp) != NULL;
+      g_free (tmp);
+    }
+
+  if (gen_cheader && have_std_marshaller)
+    {
+      fprintf (fout, "#define %s_%s\t%s_%s\n", marshaller_prefix, signame, std_marshaller_prefix, signame);
+    }
+  if (gen_cheader && !have_std_marshaller)
     {
       ind = fprintf (fout, "extern void ");
       ind += fprintf (fout, "%s_%s (", marshaller_prefix, signame);
@@ -262,7 +319,7 @@ generate_marshal (const gchar *signame,
       fprintf (fout, "%sgpointer      invocation_hint,\n", indent (ind));
       fprintf (fout, "%sgpointer      marshal_data);\n", indent (ind));
     }
-  if (gen_cbody)
+  if (gen_cbody && !have_std_marshaller)
     {
       /* cfile marhsal header */
       fprintf (fout, "void\n");
@@ -275,41 +332,41 @@ generate_marshal (const gchar *signame,
       fprintf (fout, "%sgpointer      marshal_data)\n", indent (ind));
       fprintf (fout, "{\n");
 
-      /* cfile GSignalFunc typedef */
-      ind = fprintf (fout, "  typedef %s (*GSignalFunc_%s) (", sig->rarg->cname, signame);
+      /* cfile GMarshalFunc typedef */
+      ind = fprintf (fout, "  typedef %s (*GMarshalFunc_%s) (", sig->rarg->ctype, signame);
       fprintf (fout, "%s data1,\n", pad ("gpointer"));
       for (a = 1, node = sig->args; node; node = node->next)
 	{
-	  Argument *arg = node->data;
+	  InArgument *iarg = node->data;
 
-	  if (arg->func)
-	    fprintf (fout, "%s%s arg_%d,\n", indent (ind), pad (arg->cname), a++);
+	  if (iarg->getter)
+	    fprintf (fout, "%s%s arg_%d,\n", indent (ind), pad (iarg->ctype), a++);
 	}
       fprintf (fout, "%s%s data2);\n", indent (ind), pad ("gpointer"));
 
       /* cfile marshal variables */
-      fprintf (fout, "  register GSignalFunc_%s callback;\n", signame);
+      fprintf (fout, "  register GMarshalFunc_%s callback;\n", signame);
       fprintf (fout, "  register GCClosure *cc = (GCClosure*) closure;\n");
       fprintf (fout, "  register gpointer data1, data2;\n");
-      if (sig->rarg->func)
-	fprintf (fout, "  %s v_return;\n", sig->rarg->cname);
+      if (sig->rarg->setter)
+	fprintf (fout, "  %s v_return;\n", sig->rarg->ctype);
 
-      if (sig->args || sig->rarg->func)
+      if (sig->args || sig->rarg->setter)
 	{
 	  fprintf (fout, "\n");
 
-	  if (sig->rarg->func)
+	  if (sig->rarg->setter)
 	    fprintf (fout, "  g_return_if_fail (return_value != NULL);\n");
 	  if (sig->args)
 	    {
 	      for (a = 0, node = sig->args; node; node = node->next)
 		{
-		  Argument *arg = node->data;
+		  InArgument *iarg = node->data;
 
-		  if (arg->func)
+		  if (iarg->getter)
 		    a++;
 		}
-	      fprintf (fout, "  g_return_if_fail (n_param_values >= %u);\n", 1 + a);
+	      fprintf (fout, "  g_return_if_fail (n_param_values == %u);\n", 1 + a);
 	    }
 	}
 
@@ -317,31 +374,44 @@ generate_marshal (const gchar *signame,
       fprintf (fout, "\n");
       fprintf (fout, "  if (G_CCLOSURE_SWAP_DATA (closure))\n    {\n");
       fprintf (fout, "      data1 = closure->data;\n");
-      fprintf (fout, "      data2 = g_value_get_as_pointer (param_values + 0);\n");
+      fprintf (fout, "      data2 = g_value_peek_pointer (param_values + 0);\n");
       fprintf (fout, "    }\n  else\n    {\n");
-      fprintf (fout, "      data1 = g_value_get_as_pointer (param_values + 0);\n");
+      fprintf (fout, "      data1 = g_value_peek_pointer (param_values + 0);\n");
       fprintf (fout, "      data2 = closure->data;\n");
       fprintf (fout, "    }\n");
-      fprintf (fout, "  callback = (GSignalFunc_%s) (marshal_data ? marshal_data : cc->callback);\n", signame);
+      fprintf (fout, "  callback = (GMarshalFunc_%s) (marshal_data ? marshal_data : cc->callback);\n", signame);
 
       /* cfile marshal callback action */
       fprintf (fout, "\n");
-      ind = fprintf (fout, " %s callback (", sig->rarg->func ? " v_return =" : "");
+      ind = fprintf (fout, " %s callback (", sig->rarg->setter ? " v_return =" : "");
       fprintf (fout, "data1,\n");
       for (a = 1, node = sig->args; node; node = node->next)
 	{
-	  Argument *arg = node->data;
+	  InArgument *iarg = node->data;
 
-	  if (arg->func)
-	    fprintf (fout, "%sg_value_get_%s (param_values + %d),\n", indent (ind), arg->func, a++);
+	  if (iarg->getter)
+	    fprintf (fout, "%s%s (param_values + %d),\n", indent (ind), iarg->getter, a++);
 	}
       fprintf (fout, "%sdata2);\n", indent (ind));
 
       /* cfile marshal return value storage */
-      if (sig->rarg->func)
+      if (sig->rarg->setter)
 	{
 	  fprintf (fout, "\n");
-	  fprintf (fout, "  g_value_set_%s (return_value, v_return);\n", sig->rarg->func);
+	  fprintf (fout, "  %s (return_value, v_return);\n", sig->rarg->setter);
+	  if (sig->rarg->release)
+	    {
+	      if (sig->rarg->release_check)
+		{
+		  fprintf (fout, "  if (%s (v_return))\n", sig->rarg->release_check);
+		  fprintf (fout, "    %s (v_return);\n", sig->rarg->release);
+			   
+		}
+	      else
+		{
+		  fprintf (fout, "  %s (v_return);\n", sig->rarg->release);
+		}
+	    }
 	}
 
       /* cfile marshal footer */
@@ -352,79 +422,90 @@ generate_marshal (const gchar *signame,
 static void
 process_signature (Signature *sig)
 {
-  gchar *pname, *sname;
+  gchar *pname, *sname, *tmp;
   GList *node;
 
   /* lookup and complete info on arguments */
-  if (!complete_arg (sig->rarg, TRUE))
+  if (!complete_out_arg (sig->rarg))
     {
-      g_warning ("unknown type: %s", sig->rarg->pname);
+      g_warning ("unknown type: %s", sig->rarg->keyword);
       return;
     }
   for (node = sig->args; node; node = node->next)
     {
-      Argument *arg = node->data;
+      InArgument *iarg = node->data;
 
-      if (!complete_arg (arg, FALSE))
+      if (!complete_in_arg (iarg))
 	{
-	  g_warning ("unknown type: %s", arg->pname);
+	  g_warning ("unknown type: %s", iarg->keyword);
 	  return;
 	}
     }
 
   /* construct requested marshaller name and technical marshaller name */
-  pname = g_strconcat (sig->rarg->pname, "_", NULL);
-  sname = g_strconcat (sig->rarg->sname, "_", NULL);
+  pname = g_strconcat (sig->rarg->keyword, "_", NULL);
+  sname = g_strconcat (sig->rarg->sig_name, "_", NULL);
   for (node = sig->args; node; node = node->next)
     {
-      Argument *arg = node->data;
+      InArgument *iarg = node->data;
       gchar *tmp;
 
       tmp = sname;
-      sname = g_strconcat (tmp, "_", arg->sname, NULL);
+      sname = g_strconcat (tmp, "_", iarg->sig_name, NULL);
       g_free (tmp);
       tmp = pname;
-      pname = g_strconcat (tmp, "_", arg->pname, NULL);
+      pname = g_strconcat (tmp, "_", iarg->keyword, NULL);
       g_free (tmp);
     }
 
   /* introductionary comment */
-  fprintf (fout, "\n/* %s", sig->rarg->pname);
+  fprintf (fout, "\n/* %s", sig->rarg->keyword);
   for (node = sig->args; node; node = node->next)
     {
-      Argument *arg = node->data;
+      InArgument *iarg = node->data;
 
-      fprintf (fout, "%c%s", node->prev ? ',' : ':', arg->pname);
+      fprintf (fout, "%c%s", node->prev ? ',' : ':', iarg->keyword);
     }
   if (!skip_ploc)
     fprintf (fout, " (%s)", sig->ploc);
   fprintf (fout, " */\n");
 
-  /* generate signature marshaller */
+  /* ensure technical marshaller exists (<marshaller_prefix>_<sname>) */
   generate_marshal (sname, sig);
 
-  /* put out marshaler alias if required */
-  if (gen_cheader && !g_hash_table_lookup (marshallers, pname))
+  /* put out marshaller alias for requested name if required (<marshaller_prefix>_<pname>) */
+  tmp = g_strconcat (marshaller_prefix, "_", pname, NULL);
+  if (gen_cheader && !g_hash_table_lookup (marshallers, tmp))
     {
-      gchar *tmp = g_strdup (pname);
-
       fprintf (fout, "#define %s_%s\t%s_%s\n", marshaller_prefix, pname, marshaller_prefix, sname);
 
       g_hash_table_insert (marshallers, tmp, tmp);
     }
+  else
+    g_free (tmp);
 
   g_free (pname);
   g_free (sname);
 }
 
-static Argument*
-new_arg (const gchar *pname)
+static InArgument*
+new_in_arg (const gchar *pname)
 {
-  Argument *arg = g_new0 (Argument, 1);
+  InArgument *iarg = g_new0 (InArgument, 1);
 
-  arg->pname = g_strdup (pname);
+  iarg->keyword = g_strdup (pname);
 
-  return arg;
+  return iarg;
+}
+
+static OutArgument*
+new_out_arg (const gchar *pname)
+{
+  OutArgument *oarg = g_new0 (OutArgument, 1);
+
+  oarg->keyword = g_strdup (pname);
+
+  return oarg;
 }
 
 static guint
@@ -434,7 +515,7 @@ parse_line (GScanner  *scanner,
   /* parse identifier for return value */
   if (g_scanner_get_next_token (scanner) != G_TOKEN_IDENTIFIER)
     return G_TOKEN_IDENTIFIER;
-  sig->rarg = new_arg (scanner->value.v_identifier);
+  sig->rarg = new_out_arg (scanner->value.v_identifier);
 
   /* keep a note on the location */
   sig->ploc = g_strdup_printf ("%s:%u", scanner->input_name, scanner->line);
@@ -446,7 +527,7 @@ parse_line (GScanner  *scanner,
   /* parse first argument */
   if (g_scanner_get_next_token (scanner) != G_TOKEN_IDENTIFIER)
     return G_TOKEN_IDENTIFIER;
-  sig->args = g_list_append (sig->args, new_arg (scanner->value.v_identifier));
+  sig->args = g_list_append (sig->args, new_in_arg (scanner->value.v_identifier));
 
   /* parse rest of argument list */
   while (g_scanner_peek_next_token (scanner) == ',')
@@ -457,7 +538,7 @@ parse_line (GScanner  *scanner,
       /* parse arg identifier */
       if (g_scanner_get_next_token (scanner) != G_TOKEN_IDENTIFIER)
 	return G_TOKEN_IDENTIFIER;
-      sig->args = g_list_append (sig->args, new_arg (scanner->value.v_identifier));
+      sig->args = g_list_append (sig->args, new_in_arg (scanner->value.v_identifier));
     }
   
   /* expect end of line, done */
@@ -482,6 +563,9 @@ int
 main (int   argc,
       char *argv[])
 {
+  const gchar *gruntime_marshallers[] = {
+#include	"gmarshal.strings"
+  };
   GScanner *scanner;
   GSList *slist, *files = NULL;
   gint i;
@@ -502,6 +586,24 @@ main (int   argc,
   fout = stdout;
   marshallers = g_hash_table_new (g_str_hash, g_str_equal);
 
+  /* add GRuntime standard marshallers */
+  if (std_includes)
+    for (i = 0; i < sizeof (gruntime_marshallers) / sizeof (gruntime_marshallers[0]); i++)
+      {
+	gchar *tmp = g_strdup (gruntime_marshallers[i]);
+	
+	g_hash_table_insert (marshallers, tmp, tmp);
+      }
+
+  /* put out initial heading */
+  fprintf (fout, "\n");
+  if (gen_cheader)
+    {
+      if (std_includes)
+	fprintf (fout, "#include\t<gobject/gmarshal.h>\n\n");
+      fprintf (fout, "G_BEGIN_DECLS\n");
+    }
+
   /* process input files */
   for (slist = files; slist; slist = slist->next)
     {
@@ -551,14 +653,14 @@ main (int   argc,
 		/* clean up signature contents */
 		g_free (signature.ploc);
 		if (signature.rarg)
-		  g_free (signature.rarg->pname);
+		  g_free (signature.rarg->keyword);
 		g_free (signature.rarg);
 		for (node = signature.args; node; node = node->next)
 		  {
-		    Argument *arg = node->data;
+		    InArgument *iarg = node->data;
 		    
-		    g_free (arg->pname);
-		    g_free (arg);
+		    g_free (iarg->keyword);
+		    g_free (iarg);
 		  }
 		g_list_free (signature.args);
 	      }
@@ -579,6 +681,13 @@ main (int   argc,
       close (fd);
     }
 
+  /* put out trailer */
+  if (gen_cheader)
+    {
+      fprintf (fout, "\nG_END_DECLS\n");
+    }
+  fprintf (fout, "\n");
+
   /* clean up */
   g_slist_free (files);
   g_scanner_destroy (scanner);
@@ -613,6 +722,16 @@ parse_args (gint    *argc_p,
 	  skip_ploc = TRUE;
 	  argv[i] = NULL;
 	}
+      else if (strcmp ("--nostdinc", argv[i]) == 0)
+	{
+	  std_includes = FALSE;
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--stdinc", argv[i]) == 0)
+	{
+	  std_includes = TRUE;
+	  argv[i] = NULL;
+	}
       else if ((strcmp ("--prefix", argv[i]) == 0) ||
 	       (strncmp ("--prefix=", argv[i], 9) == 0))
 	{
@@ -690,12 +809,13 @@ print_blurb (FILE    *bout,
   else
     {
       fprintf (bout, "Usage: %s [options] [files...]\n", PRG_NAME);
-      fprintf (bout, "  --header                        generate C headers\n");
-      fprintf (bout, "  --body                          generate C code\n");
-      fprintf (bout, "  --prefix=string                 specify marshaller prefix\n");
-      fprintf (bout, "  --skip-source                   skip source location comments\n");
-      fprintf (bout, "  -h, --help                      show this help message\n");
-      fprintf (bout, "  -v, --version                   print version informations\n");
-      fprintf (bout, "  --g-fatal-warnings              make warnings fatal (abort)\n");
+      fprintf (bout, "  --header                   generate C headers\n");
+      fprintf (bout, "  --body                     generate C code\n");
+      fprintf (bout, "  --prefix=string            specify marshaller prefix\n");
+      fprintf (bout, "  --skip-source              skip source location comments\n");
+      fprintf (bout, "  --stdinc, --nostdinc       include/use GRuntime standard marshallers\n");
+      fprintf (bout, "  -h, --help                 show this help message\n");
+      fprintf (bout, "  -v, --version              print version informations\n");
+      fprintf (bout, "  --g-fatal-warnings         make warnings fatal (abort)\n");
     }
 }
