@@ -99,6 +99,9 @@ static GStaticRWLock            type_rw_lock = G_STATIC_RW_LOCK_INIT;
 				    G_TYPE_FLAG_DERIVABLE | \
 				    G_TYPE_FLAG_DEEP_DERIVABLE)
 #define	TYPE_FLAG_MASK		   (G_TYPE_FLAG_ABSTRACT | G_TYPE_FLAG_VALUE_ABSTRACT)
+#define	SIZEOF_FUNDAMENTAL_INFO	   ((gssize) MAX (MAX (sizeof (GTypeFundamentalInfo), \
+						       sizeof (gpointer)), \
+                                                  sizeof (glong)))
 
 
 /* --- typedefs --- */
@@ -287,12 +290,12 @@ type_node_any_new_W (TypeNode             *pnode,
     static_type_nodes[ftype] = g_renew (TypeNode*, static_type_nodes[ftype], 1 << g_bit_storage (static_branch_seqnos[ftype] - 1));
   
   if (!pnode)
-    node_size += sizeof (GTypeFundamentalInfo);	      /* fundamental type info */
+    node_size += SIZEOF_FUNDAMENTAL_INFO;	      /* fundamental type info */
   node_size += SIZEOF_BASE_TYPE_NODE ();	      /* TypeNode structure */
   node_size += (sizeof (GType) * (1 + n_supers + 1)); /* self + ancestors + 0 for ->supers[] */
   node = g_malloc0 (node_size);
   if (!pnode)					      /* offset fundamental types */
-    node = G_STRUCT_MEMBER_P (node, sizeof (GTypeFundamentalInfo));
+    node = G_STRUCT_MEMBER_P (node, SIZEOF_FUNDAMENTAL_INFO);
   static_type_nodes[ftype][branch_last] = node;
   
   node->n_supers = n_supers;
@@ -362,7 +365,7 @@ type_node_fundamental_info_L (TypeNode *node)
   if (ftype != NODE_TYPE (node))
     node = lookup_type_node_L (ftype);
   
-  return node ? G_STRUCT_MEMBER_P (node, - (gssize) sizeof (GTypeFundamentalInfo)) : NULL;
+  return node ? G_STRUCT_MEMBER_P (node, - SIZEOF_FUNDAMENTAL_INFO) : NULL;
 }
 
 static TypeNode*
@@ -448,8 +451,8 @@ type_lookup_iface_entry_L (TypeNode *node,
 }
 
 static inline gboolean
-type_lookup_prerequisite (TypeNode *iface,
-			  GType     prerequisite_type)
+type_lookup_prerequisite_L (TypeNode *iface,
+			    GType     prerequisite_type)
 {
   if (NODE_IS_IFACE (iface) && IFACE_NODE_N_PREREQUISITES (iface))
     {
@@ -814,13 +817,14 @@ check_add_interface_L (GType instance_type,
   tnode = lookup_type_node_L (NODE_PARENT_TYPE (iface));
   if (NODE_PARENT_TYPE (tnode) && !type_lookup_iface_entry_L (node, tnode))
     {
+      /* 2001/7/31:timj: erk, i guess this warning is junk as interface derivation is flat */
       g_warning ("cannot add sub-interface `%s' to type `%s' which does not conform to super-interface `%s'",
 		 NODE_NAME (iface),
 		 NODE_NAME (node),
 		 NODE_NAME (tnode));
       return FALSE;
     }
-  tnode = find_conforming_type_L (node, iface);
+  tnode = find_conforming_type_L (node, iface);  // FIXME: iface overriding
   if (tnode)
     {
       g_warning ("cannot add interface type `%s' to type `%s', since type `%s' already conforms to interface",
@@ -1163,7 +1167,8 @@ g_type_interface_add_prerequisite (GType interface_type,
   if (prerequisite_node->is_instantiatable)
     {
       guint i;
-      
+
+      /* can have at most one publically installable instantiatable prerequisite */
       for (i = 0; i < IFACE_NODE_N_PREREQUISITES (iface); i++)
 	{
 	  TypeNode *prnode = lookup_type_node_L (IFACE_NODE_PREREQUISITES (iface)[i]);
@@ -1194,7 +1199,7 @@ g_type_interface_add_prerequisite (GType interface_type,
       type_iface_add_prerequisite_W (iface, prerequisite_node);
     }
   else
-    g_warning ("prerequisite `%s' for interface `%s' is not instantiatable or interface",
+    g_warning ("prerequisite `%s' for interface `%s' is neither instantiatable nor interface",
 	       type_descriptive_name_L (prerequisite_type),
 	       type_descriptive_name_L (interface_type));
   G_WRITE_UNLOCK (&type_rw_lock);
@@ -1388,7 +1393,9 @@ static void
 type_iface_vtable_init_Wm (TypeNode *iface,
 			   TypeNode *node)
 {
+#ifndef G_DISABLE_ASSERT
   IFaceEntry *entry = type_lookup_iface_entry_L (node, iface);
+#endif
   IFaceHolder *iholder = type_iface_retrive_holder_info_Wm (iface, NODE_TYPE (node));
   GTypeInterface *vtable;
   
@@ -1415,7 +1422,9 @@ type_iface_vtable_finalize_Wm (TypeNode       *iface,
 			       TypeNode       *node,
 			       GTypeInterface *vtable)
 {
+#ifndef G_DISABLE_ASSERT
   IFaceEntry *entry = type_lookup_iface_entry_L (node, iface);
+#endif
   IFaceHolder *iholder = iface_node_get_holders_L (iface);
   
   g_assert (entry && entry->vtable == vtable);
@@ -2070,6 +2079,18 @@ g_type_parent (GType type)
   return node ? NODE_PARENT_TYPE (node) : 0;
 }
 
+guint
+g_type_depth (GType type)
+{
+  TypeNode *node;
+  
+  G_READ_LOCK (&type_rw_lock);
+  node = lookup_type_node_L (type);
+  G_READ_UNLOCK (&type_rw_lock);
+  
+  return node ? node->n_supers + 1 : 0;
+}
+
 GType
 g_type_next_base (GType type,
 		  GType base_type)
@@ -2113,7 +2134,7 @@ type_node_is_a_L (TypeNode *node,
     return TRUE;
   else if (support_prerequisites &&
 	   NODE_IS_IFACE (node) &&
-	   type_lookup_prerequisite (node, NODE_TYPE (iface_node)))
+	   type_lookup_prerequisite_L (node, NODE_TYPE (iface_node)))
     return TRUE;
   else
     return FALSE;
@@ -2685,7 +2706,7 @@ g_type_value_table_peek (GType type)
   node = lookup_type_node_L (type);
   if (!node)
     {
-      g_warning ("type id `%u' is invalid", type);
+      g_warning (G_STRLOC ": type id `%u' is invalid", type);
       G_READ_UNLOCK (&type_rw_lock);
       return NULL;
     }
@@ -2722,6 +2743,24 @@ g_type_value_table_peek (GType type)
   return vtable;
 }
 
+G_CONST_RETURN gchar*
+g_type_name_from_instance (GTypeInstance *instance)
+{
+  if (!instance)
+    return "<NULL-instance>";
+  else
+    return g_type_name_from_class (instance->g_class);
+}
+
+G_CONST_RETURN gchar*
+g_type_name_from_class (GTypeClass *g_class)
+{
+  if (!g_class)
+    return "<NULL-class>";
+  else
+    return g_type_name (g_class->g_type);
+}
+
 
 /* --- foreign prototypes --- */
 extern void	g_value_types_init	(void); /* sync with gvaluetypes.c */
@@ -2736,7 +2775,7 @@ extern void	g_signal_init		(void);	/* sync with gsignal.c */
 
 /* --- initialization --- */
 void
-g_type_init (GTypeDebugFlags debug_flags)
+g_type_init_with_debug_flags (GTypeDebugFlags debug_flags)
 {
   G_LOCK_DEFINE_STATIC (type_init_lock);
   static TypeNode *type0_node = NULL;
@@ -2842,3 +2881,9 @@ g_type_init (GTypeDebugFlags debug_flags)
   
   G_UNLOCK (type_init_lock);
 }
+
+void 
+g_type_init (void)
+{
+  g_type_init_with_debug_flags (0);
+}
